
##############################################################################################
# Function to simulate a data set since the Logistic model and                               #
# fit using Logistic model using glm (Classical inference) and MCMCpack (bayesian inference) #
##############################################################################################


modeloLogist<- function(n,beta){

#function to generate Logistic data
geraL<-function(n,beta)
{
n<-n
beta1<-beta[2]
beta0<-beta[1]
#set.seed(123)
x<-matrix(runif(n,-3,3),ncol=1)
pL<-plogis(beta0+x%*%beta1)
#u<-matrix(runif(n,min=-1, max=1),n,1)
u<-matrix(runif(n),n,1)
y<-ifelse(u<pL,1,0)
return(cbind(y,x,pL))
}


#generating data for glm
n=1000
beta=c(0,1)

dataall<-geraL(n,beta)
dataL<-dataall[,c(1,2)]
p<-dataall[,3]
minL=min(p)
maxL=max(p)
minL
maxL
mean(p)

plot(dataL[,2],p)

#RangoL=data.frame(minL,maxL)
dataLglm=data.frame(dataL)
names(dataLglm)<-c("y","x")

#fit using ML estimation
fitLglm=glm(y ~ x, family = binomial(link=logit), data = dataLglm)
fitLglm
betafitLglm<-coef(fitLglm)
betafitLglm


#Fit Bayesian using MCMCpack

y=dataLglm$y
x=dataLglm$x

require(MCMCpack) 
fitLMCMC=MCMClogit(y~x)

#plot(fitLMCMC)
model.bayes=summary(fitLMCMC)
model.bayes
a=model.bayes$statistics
a=data.frame(a)
attach(a)
betafitLMCMC=c(a$Mean[1],a$Mean[2])

#final output
output<-c(n,betafitLglm,betafitLMCMC,minL,maxL)

return(output)
}

##############################################################################################
# Function to simulate datasets with replications since the Logistic model and               #
# fit using Logistic model using glm (Classical inference) and MCMCpack (bayesian inference) #
##############################################################################################


replicLogist<-function(M,n,beta)
{
replica=matrix(0,M,7)
colnames(replica)<-c("size","Beta_0Lglm","Beta_1Lglm","Beta_0LMCMC","Beta_1MCMC","MinimoL","MaximoL")
for(i in 1:M){
replica[i,]=modeloLogist(n,beta)
}

meanLa=mean(replica[,2])
meanLb=mean(replica[,3])
meanMa=mean(replica[,4])
meanMb=mean(replica[,5])
biasLa=mean(replica[,2])-beta[1]
biasLb=mean(replica[,3])-beta[2]
biasMa=mean(replica[,4])-beta[1]
biasMb=mean(replica[,5])-beta[2]
sdLa=sd(replica[,2])/sqrt(M)
sdLb=sd(replica[,3])/sqrt(M)
sdMa=sd(replica[,4])/sqrt(M)
sdMb=sd(replica[,5])/sqrt(M)
mseLa<-mean((replica[,2]-beta[1])^2)
mseLb<-mean((replica[,3]-beta[2])^2)
mseMa<-mean((replica[,4]-beta[1])^2)
mseMb<-mean((replica[,5]-beta[2])^2)
meanLmin=mean(replica[,6])
meanLmax=mean(replica[,7])


resmean=c(meanLa,meanLb,meanMa,meanMb)
names(resmean)<-c("meanLa","meanLb","meanMa","meanMb")
resbias=c(biasLa,biasLb,biasMa,biasMb)
names(resbias)<-c("biasLa","biasLb","biasMa","biasMb")
ressd=c(sdLa,sdLb,sdMa,sdMb)
names(ressd)<-c("sdLa","sdLb","sdMa","sdMb")
resmse=c(mseLa,mseLb,mseMa,mseMb)
names(resmse)<-c("mseLa","mseLb","mseMa","mseMb")
rango=c(meanLmin,meanLmax)
names(rango)<-c("MinL","MaxL")
b<-list(repl=replica,mean=resmean,bias=resbias,sd=ressd,mse=resmse,rango=rango)
print(b)
}

M=10
n=1000
beta=c(0,1)
replicLogist(M,n,beta)

#Homework
#########
#Generate dataset since a Power Logistic and fit using Logistic

