
##############################################################################################
# Function to simulate a data set since the Power Probit model and                           #
# fit using Probit model using glm (Classical inference) and MCMCpack (bayesian inference)   #
##############################################################################################


modeloPP<- function(n,beta,lambda){

#function to generate Power probit data
geraPP<-function(n,beta,lambda)
{
n<-n
beta1<-beta[2]
beta0<-beta[1]
#set.seed(123)
x<-matrix(runif(n,-3,3),ncol=1)
pL1<-pnorm(beta0+x%*%beta1)
pL<-pL1**lambda
#u<-matrix(runif(n,min=-1, max=1),n,1)
u<-matrix(runif(n),n,1)
y<-ifelse(u<pL,1,0)
return(cbind(y,x,pL))
}

#generating dataset
n=1000
beta=c(0,1)
lambda=1/3

dataall<-geraPP(n,beta,lambda)
dataL<-dataall[,c(1,2)]
p<-dataall[,3]
minL=min(p)
maxL=max(p)
meanp=mean(p)

plot(dataL[,2],p)

#RangoL=data.frame(minL,maxL)
dataLglm=data.frame(dataL)
names(dataLglm)<-c("y","x")

#fit using ML estimation
fitLglm=glm(y ~ x, family = binomial(link=probit), data = dataLglm)
fitLglm
betafitLglm<-coef(fitLglm)
betafitLglm


#Fit Bayesian using MCMCpack

y=dataLglm$y
x=dataLglm$x

require(MCMCpack) 
fitLMCMC=MCMCprobit(y~x)

#plot(fitLMCMC)
model.bayes=summary(fitLMCMC)
model.bayes
a=model.bayes$statistics
a=data.frame(a)
attach(a)
betafitLMCMC=c(a$Mean[1],a$Mean[2])

#final output
output<-c(n,betafitLglm,betafitLMCMC,minL,maxL,meanp)

return(output)
}

##############################################################################################
# Function to simulate datasets with replications since the Power Probit model and           #
# fit using Probit model using glm (Classical inference) and MCMCpack (bayesian inference)   #
##############################################################################################


replicPP<-function(M,n,beta,lambda)
{
replica=matrix(0,M,8)
colnames(replica)<-c("size","Beta_0Lglm","Beta_1Lglm","Beta_0LMCMC","Beta_1MCMC","Minimop","Maximop","Mediap")
for(i in 1:M){
replica[i,]=modeloPL(n,beta,lambda)
}

meanLa=mean(replica[,2])
meanLb=mean(replica[,3])
meanMa=mean(replica[,4])
meanMb=mean(replica[,5])
biasLa=mean(replica[,2])-beta[1]
biasLb=mean(replica[,3])-beta[2]
biasMa=mean(replica[,4])-beta[1]
biasMb=mean(replica[,5])-beta[2]
sdLa=sd(replica[,2])/sqrt(M)
sdLb=sd(replica[,3])/sqrt(M)
sdMa=sd(replica[,4])/sqrt(M)
sdMb=sd(replica[,5])/sqrt(M)
mseLa<-mean((replica[,2]-beta[1])^2)
mseLb<-mean((replica[,3]-beta[2])^2)
mseMa<-mean((replica[,4]-beta[1])^2)
mseMb<-mean((replica[,5]-beta[2])^2)
meanpmin=mean(replica[,6])
meanpmax=mean(replica[,7])
meanp=mean(replica[,8])

resmean=c(meanLa,meanLb,meanMa,meanMb)
names(resmean)<-c("meanLa","meanLb","meanMa","meanMb")
resbias=c(biasLa,biasLb,biasMa,biasMb)
names(resbias)<-c("biasLa","biasLb","biasMa","biasMb")
ressd=c(sdLa,sdLb,sdMa,sdMb)
names(ressd)<-c("sdLa","sdLb","sdMa","sdMb")
resmse=c(mseLa,mseLb,mseMa,mseMb)
names(resmse)<-c("mseLa","mseLb","mseMa","mseMb")
rango=c(meanpmin,meanpmax,meanp)
names(rango)<-c("Minp","Maxp","Meanp")
b<-list(repl=replica,mean=resmean,bias=resbias,sd=ressd,mse=resmse,rango=rango)
print(b)
}

M=10
n=1000
beta=c(0,1)
lambda=4
replicPP(M,n,beta,lambda)




